import pytest
from lotterybr import get_data, InvalidGameError, InvalidTypeError


def test_get_data_invalid_game():
    with pytest.raises(InvalidGameError):
        get_data('invalidgame', 'numbers')

def test_get_data_invalid_type():
    with pytest.raises(InvalidTypeError):
        get_data('megasena', 'invalidtype')

    
if __name__ == '__main__':
    pytest.main()


from shiny import App, render, ui, reactive
from shinywidgets import output_widget, register_widget
import plotly.express as px
import pandas as pd
import numpy as np

from .get_data import get_data

descriptions = {
    'maismilionaria' : "Para ganhar na MaisMilionaria, e necessario acertar pelo menos quatro dos seis numeros sorteados.",
    'megasena' : "Na Mega-Sena, ha diversas faixas de premiacao. Para ganhar o premio maximo (Sena), e preciso acertar todos os seis numeros sorteados. Ha tambem premiacoes para quem acerta cinco numeros (Quina) e quatro numeros (Quadra).",
    'lotofacil' : "Na Lotofacil, existem diferentes faixas de premiacao. Para ganhar o premio maximo, e necessario acertar os quinze numeros sorteados. Ha tambem premiacoes para quem acerta onze, doze, treze ou quatorze numeros.",
    'quina' : "Na Quina, existem diferentes faixas de premiacao. Para ganhar o premio maximo, e necessario acertar os cinco numeros sorteados. Ha tambem premiacoes para quem acerta dois, tres ou quatro numeros.",
    'lotomania' : "Na Lotomania, ha diversas faixas de premiacao. Para ganhar o premio maximo, e necessario acertar todos os vinte numeros sorteados. Alem disso, ha premiacoes para quem acerta dezesseis, dezessete, dezoito ou dezenove numeros.",
    'duplasena' : "Na Dupla Sena, existem diferentes faixas de premiacao. Para ganhar o premio maximo (Sena), e necessario acertar os seis numeros sorteados no primeiro ou no segundo sorteio. Ha tambem premiacoes para quem acerta cinco (Quina) ou quatro (Quadra) numeros em um dos sorteios.",
    'diadesorte' : "No Dia de Sorte, e preciso acertar sete numeros sorteados mais o mes para ganhar o premio maximo. Ha tambem premiacoes para quem acerta seis, cinco, quatro ou tres numeros, independente do mes."
}

app_ui = ui.page_fluid(
    ui.tags.style(
        """
        .data-table-container {
            max-height: 300px;
            overflow-y: auto;
        }
        """
    ),
    ui.h2("Aplicativo Lotterybr"),
    ui.layout_sidebar(
        ui.page_sidebar(
            ui.input_select("jogo", "Escolha o Jogo:",
                            {"maismilionaria": "MaisMilionária", "megasena": "Mega-Sena", "lotofacil": "Lotofácil", 
                             "quina": "Quina", "lotomania": "Lotomania", "duplasena": "Dupla Sena", "diadesorte": "Dia de Sorte"}),
            ui.input_select("tipo", "Escolha o tipo de dados", {"numbers": "Números", "winners": "Ganhadores"}),
            ui.input_select("grafico", "Escolha o tipo de gráfico", {"bar_chart": "Gráfico de Barras"}),
            ui.panel_conditional(
                "input.tipo == 'winners'",
                ui.input_checkbox("log_scale", "Usar escala logarítmica", False)
            ),
            ui.output_text_verbatim("summary_table")
        ),
        ui.panel_main(
            output_widget("plot"),
            ui.h5("Descrição: "),
            ui.output_text("dynamic_text"),
            ui.h3("Tabela de Dados"),
            ui.div(ui.output_table("data_table"), class_="data-table-container")
        )
    )
)

def server(input, output, session):

    @reactive.Effect
    @reactive.event(input.jogo, input.tipo, input.grafico)
    def _():
        game = input.jogo()
        tipo = input.tipo()
        dados = get_data(game, tipo, language= "en")

        if tipo == "numbers":
            if game == "maismilionaria":
                resultado = [x for x in dados['numbers_clovers'] if x.isdigit()]
                df = pd.DataFrame({'Número': resultado}).value_counts().reset_index(name='Frequência')
                fig = px.bar(df, x='Número', y='Frequência', title="Frequência de Números da MaisMilionária")
            else:
                df = pd.DataFrame({'Número': dados['numbers']}).value_counts().reset_index(name='Frequência')
                fig = px.bar(df, x='Número', y='Frequência', title=f"Frequência de Números da {game.capitalize()}")
                
        elif tipo == "winners":
            df = pd.DataFrame(dados)
            if input.log_scale():
                df['winners'] = df['winners'].apply(lambda x: np.log1p(x))
            fig = px.bar(df, x='match', y='winners', title=f"Frequência de Ganhadores da {game.capitalize()}")

        register_widget("plot", fig)

    @output
    @render.text
    def dynamic_text():
        return descriptions[input.jogo()]

    @output
    @render.table
    def data_table():
        game = input.jogo()
        tipo = input.tipo()
        dados = get_data(game, tipo, language="eng")
        return pd.DataFrame(dados)

    @output
    @render.text
    def summary_table():
        game = input.jogo()
        tipo = input.tipo()
        dados = get_data(game, tipo, language="eng")
        return pd.DataFrame(dados).describe().to_string()

app = App(app_ui, server)



from .get_data import get_data, InvalidGameError, InvalidTypeError, InvalidLanguageError
from .open_app import open_app, InvalidLanguageError

__all__ = ["get_data", "open_app", "InvalidGameError", "InvalidTypeError", "InvalidLanguageError"]



from typing import Literal

from .app_eng import app as eng
from .app_ptbr import app as ptbr

Languages = Literal["eng", "ptbr"]

class InvalidLanguageError(ValueError):
    """Raised when an invalid language is provided."""
    def __init__(self, language: str):
        super().__init__(f"Invalid type: {language}. Must be one of {', '.join(Languages.__args__)}.")

def open_app(language = "eng"):
    """
    Opens the lottery shiny in the specified language.

    This function allows users to choose between the English (eng) and Brazilian Portuguese (ptbr) versions of the lottery shiny.

    Parameters
    ----------
    language : Languages, default="eng"
        The language to use for the application. Must be one of:
            - "eng" for English
            - "ptbr" for Brazilian Portuguese

    Raises
    ----------
    InvalidLanguageError
        If an unsupported language is provided.

    Examples
    --------
    Open the application in English:

    >>> open_app()

    Open the application in Brazilian Portuguese:

    >>> open_app(language="ptbr")
    """
    valid_languages = ("eng", "ptbr")
    if language not in valid_languages:
        raise InvalidLanguageError(language)
    
    if language == "eng":
        eng.run()
    elif language == "ptbr":
        ptbr.run()

open_app()


import pandas as pd

def translateData(data, game, type):
    if game == "maismilionaria":
        if type == "ganhadores":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'match': 'acertos',
                'winners': 'ganhadores',
                'prize': 'premio'
            })
            data['acertos'] = data['acertos'].replace({
                "6 + 2 clovers": "sena + 2 trevos",
                "6 + 1 or 0 clovers": "sena + 1 ou 0 trevos",
                "5 + 2 clovers": "quina + 2 trevos",
                "5 + 1 or 0 clovers": "quina + 1 ou 0 trevos",
                "4 + 2 clovers": "quadra + 2 trevos",
                "4 + 1 or 0 clovers": "quadra + 1 ou 0 trevos",
                "3 + 2 clovers": "terno + 2 trevos",
                "3 + 1 clovers": "terno + 1 trevo",
                "2 + 2 clovers": "duque + 2 trevos",
                "2 + 1 clover": "duque + 1 trevo"
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })
        elif type == "dezenas":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'numbers_clovers': 'dezenas_trevos'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })

    elif game == "megasena":
        if type == "ganhadores":
            print("oi")
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'match': 'acertos',
                'winners': 'ganhadores',
                'prize': 'premio'
            })
            data['acertos'] = data['acertos'].replace({
                "6": "sena",
                "5": "quina",
                "4": "quadra"
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })
        elif type == "dezenas":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'numbers': 'dezenas'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })

    elif game == "lotofacil":
        if type == "ganhadores":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'match': 'acertos',
                'winners': 'ganhadores',
                'prize': 'premio'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })
        elif type == "dezenas":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'numbers': 'dezenas'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })

    elif game == "quina":
        if type == "ganhadores":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'match': 'acertos',
                'winners': 'ganhadores',
                'prize': 'premio'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })
        elif type == "dezenas":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'numbers': 'dezenas'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })

    elif game == "lotomania":
        if type == "ganhadores":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'match': 'acertos',
                'winners': 'ganhadores',
                'prize': 'premio'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })
        elif type == "dezenas":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'numbers': 'dezenas'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })

    elif game == "duplasena":
        if type == "ganhadores":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'match': 'acertos',
                'winners': 'ganhadores',
                'prize': 'premio'
            })
            data['acertos'] = data['acertos'].replace({
                "6": "sena",
                "5": "quina",
                "4": "quadra",
                "3": "terno"
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })
        elif type == "dezenas":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'numbers1': 'dezenas1',
                'numbers2': 'dezenas2'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })

    elif game == "diadesorte":
        if type == "ganhadores":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'match': 'acertos',
                'winners': 'ganhadores',
                'prize': 'premio'
            })
            data['acertos'] = data['acertos'].replace({
                "Lucky Month": "Mes da Sorte"
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })
        elif type == "dezenas":
            data = data.rename(columns={
                'date': 'data',
                'course': 'concurso',
                'accumulated': 'acumula',
                'numbers': 'dezenas',
                'month': 'mes'
            })
            data['acumula'] = data['acumula'].replace({
                "yes": "sim",
                "no": "nao"
            })
            data['mes'] = data['mes'].replace({
                "January": "Janeiro",
                "February": "Fevereiro",
                "March": "Marco",
                "April": "Abril",
                "May": "Maio",
                "June": "Junho",
                "July": "Julho",
                "August": "Agosto",
                "September": "Setembro",
                "October": "Outubro",
                "November": "Novembro",
                "December": "Dezembro"
            })

    return data



import os
from typing import Literal

import pandas as pd
import pyreadr

from .translate_data import translateData


Games = Literal["maismilionaria", "megasena", "lotofacil", "quina", "lotomania", "duplasena", "diadesorte"]
Types = Literal["numbers", "winners", "dezenas", "ganhadores"]
Languages = Literal["eng", "ptbr"]

class InvalidGameError(ValueError):
    """Raised when an invalid game is provided."""
    def __init__(self, game: str):
        super().__init__(f"Invalid game: {game}. Must be one of {', '.join(Games.__args__)}.")

class InvalidTypeError(ValueError):
    """Raised when an invalid type is provided."""
    def __init__(self, type: str):
        super().__init__(f"Invalid type: {type}. Must be one of {', '.join(Types.__args__)}.")

class InvalidLanguageError(ValueError):
    """Raised when an invalid language is provided."""
    def __init__(self, language: str):
        super().__init__(f"Invalid type: {language}. Must be one of {', '.join(Languages.__args__)}.")

def get_data(
    game: Games = "megasena", 
    type: Types = "winners",
    language: Languages = "eng",
) -> pd.DataFrame:
    """
    Retrieves lottery data for a specified game and data type.

    This function fetches data from the LotteryBrasilDATA repository on GitHub for a given lottery game
    and data type ('numbers' or 'winners').

    Parameters:
        game (Games, optional): The name of the lottery game. Defaults to "megasena".
            Allowed values:
                - 'maismilionaria': Mais Milionária
                - 'megasena': Mega-Sena
                - 'lotofacil': Lotofácil
                - 'quina': Quina
                - 'lotomania': Lotomania
                - 'duplasena': Dupla Sena
                - 'diadesorte': Dia de Sorte
        type (Types, optional): The type of data to retrieve. Defaults to "winners".
            Allowed values:
                - 'numbers': Drawn numbers data
                - 'winners': Winners data
                - 'dezenas' (Portuguese): Equivalent to 'numbers'
                - 'ganhadores' (Portuguese): Equivalent to 'winners'
        language (Languages, optional): The language for the returned data. Defaults to "eng".
            Allowed values:
                - 'eng': English
                - 'ptbr': Portuguese (Brasil)

    Returns:
        pd.DataFrame: A DataFrame containing the loaded data.

    Raises:
        InvalidGameError: If an invalid game name is provided.
        InvalidTypeError: If an invalid data type is provided.
        InvalidLanguageError: If an invalid language is provided.
        ValueError: If there's an error downloading or loading the data.

    Examples:
        Get drawn numbers data for Mega-Sena in English:

        ```python
        df = get_data('megasena', 'numbers')
        print(df.head())
        ```

        Get winners data for Lotofácil in Portuguese:

        ```python
        df = get_data('lotofacil', 'ganhadores', 'ptbr')
        print(df.head())
        ```
    """
    valid_games = {"maismilionaria", "megasena", "lotofacil", "quina", "lotomania", "duplasena", "diadesorte"}
    valid_languages = ("eng", "ptbr")
    valid_types = {"numbers", "winners", "dezenas", "ganhadores"}

    if game not in valid_games:
        raise InvalidGameError(game)

    if type not in valid_types:
        raise InvalidTypeError(type)
    
    if language not in valid_languages:
        raise InvalidLanguageError(language)
    
    match type:
        case 'numbers':
            type_str = 'dezenas'
        case 'winners':
            type_str = 'ganhadores'
        case 'dezenas':
            type_str = 'dezenas'
        case 'ganhadores':
            type_str = 'ganhadores'

    url = f"https://github.com/tomasbp2/LotteryBrasilDATA/blob/main/{game}/{type_str}.rds?raw=true"

    try:
        module_dir = os.path.dirname(os.path.abspath(__file__))
        dst_dir = os.path.join(module_dir, "..", "dataset", game)

        if not os.path.exists(dst_dir):
            os.makedirs(dst_dir)

        dst_path = os.path.join(dst_dir, f"{type_str}.rds")
        pyreadr.download_file(url, dst_path)
        result = pyreadr.read_r(dst_path)
        df = result[None]
        
    except Exception as e:
        raise ValueError(f"Error while importing data: {e}")
    
    if language == 'ptbr':
        df = translateData(df, game, type)

    return df



from setuptools import setup, find_packages

with open("README.md") as f:
    README = f.read()

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Science/Research",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Topic :: Database",
]


# Read requirements from requirements.txt
REQUIREMENTS = [
    'numpy==2.1.3',
    'pandas==2.2.3',
]

PROJECT_URLS = {
    #"Bug Tracker": "https://github.com/salvnetto/pyreglin",
    #"Documentation": "https://salvnetto.github.io/pyreglin",
    "Source Code": "https://github.com/salvnetto/pyreglin",
}

setup(
    name='lotterybr',
    version='1.0.2',
    description= "A Python package to access and analyze Brazilian lottery data.",
    packages= find_packages(),
    long_description= README,
    long_description_content_type= "text/markdown",
    url= "https://github.com/salvnetto/lotterybr-py",
    author= "Fábio N. Demarqui",
    author_email= "fndemarqui@est.ufmg.br",
    maintainer= "Salvador Netto, Tomás Bernardes",
    maintainer_email= "salvv.netto@gmail.com",
    license= "MIT",
    platforms="any",
    classifiers= CLASSIFIERS,
    install_requires= REQUIREMENTS,
    zip_safe=False,
    python_requires='>3.8',
    project_urls=PROJECT_URLS,
)


from shiny import App, render, ui, reactive
from shinywidgets import output_widget, register_widget
import plotly.express as px
import pandas as pd
import numpy as np

from lotterybr import get_data

descriptions = {
    "maismilionaria": "To win in MaisMilionaria, you need to match at least four of the six drawn numbers.",
    "megasena": "In Mega-Sena, there are various prize tiers. To win the top prize (Sena), you must match all six drawn numbers. There are also prizes for matching five numbers (Quina) and four numbers (Quadra).",
    "lotofacil": "In Lotofacil, there are different prize tiers. To win the top prize, you need to match all fifteen drawn numbers. There are also prizes for matching eleven, twelve, thirteen, or fourteen numbers.",
    "quina": "In Quina, there are different prize tiers. To win the top prize, you need to match all five drawn numbers. There are also prizes for matching two, three, or four numbers.",
    "lotomania": "In Lotomania, there are various prize tiers. To win the top prize, you need to match all twenty drawn numbers. Additionally, there are prizes for matching sixteen, seventeen, eighteen, or nineteen numbers.",
    "duplasena": "In Dupla Sena, there are different prize tiers. To win the top prize (Sena), you need to match all six drawn numbers in either the first or the second draw. There are also prizes for matching five (Quina) or four (Quadra) numbers in one of the draws.",
    "diadesorte": "In Dia de Sorte, you need to match seven drawn numbers plus the month to win the top prize. There are also prizes for matching six, five, four, or three numbers, regardless of the month."
}

app_ui = ui.page_fluid(
    ui.tags.style(
        """
        .data-table-container {
            max-height: 300px;
            overflow-y: auto;
        }
        """
    ),
    ui.h2("Lotterybr App"),
    ui.layout_sidebar(
        ui.page_sidebar(
            ui.input_select("jogo", "Choose Game:",
                            {"maismilionaria": "MaisMilionaria", "megasena": "Mega-Sena", "lotofacil": "LotoFacil", 
                             "quina": "Quina", "lotomania": "LotoMania", "duplasena": "Dupla Sena", "diadesorte": "Dia de Sorte"}),
            ui.input_select("tipo", "Choose data type", {"numbers": "Numbers", "winners": "Winners"}),
            ui.input_select("grafico", "Choose graph type", {"bar_chart": "Bar Chart"}),
            ui.panel_conditional(
                "input.tipo == 'winners'",
                ui.input_checkbox("log_scale", "Use log scale", False)
            ),
            ui.output_text_verbatim("summary_table")
        ),
        ui.panel_main(
            output_widget("plot"),
            ui.h5("Description: "),
            ui.output_text("dynamic_text"),
            ui.h3("Data Table"),
            ui.div(ui.output_table("data_table"), class_="data-table-container")
        )
    )
)

def server(input, output, session):

    @reactive.Effect
    @reactive.event(input.jogo, input.tipo, input.grafico)
    def _():
        game = input.jogo()
        tipo = input.tipo()
        dados = get_data(game, tipo, language= "eng")

        if tipo == "numbers":
            if game == "maismilionaria":
                resultado = [x for x in dados['numbers_clovers'] if x.isdigit()]
                df = pd.DataFrame({'Number': resultado}).value_counts().reset_index(name='Frequency')
                fig = px.bar(df, x='Number', y='Frequency', title="MaisMilionaria Numbers Frequency")
            else:
                df = pd.DataFrame({'Number': dados['numbers']}).value_counts().reset_index(name='Frequency')
                fig = px.bar(df, x='Number', y='Frequency', title=f"{game.capitalize()} Numbers Frequency")
                
        elif tipo == "winners":
            df = pd.DataFrame(dados)
            if input.log_scale():
                df['winners'] = df['winners'].apply(lambda x: np.log1p(x))
            fig = px.bar(df, x='match', y='winners', title=f"{game.capitalize()} Winners Frequency")

        register_widget("plot", fig)

    @output
    @render.text
    def dynamic_text():
        return descriptions[input.jogo()]

    @output
    @render.table
    def data_table():
        game = input.jogo()
        tipo = input.tipo()
        dados = get_data(game, tipo, language="eng")
        return pd.DataFrame(dados)

    @output
    @render.text
    def summary_table():
        game = input.jogo()
        tipo = input.tipo()
        dados = get_data(game, tipo, language="eng")
        return pd.DataFrame(dados).describe().to_string()

app = App(app_ui, server)
